#![allow(unused_macros)]

use anyhow::{anyhow, bail};
use directories::ProjectDirs;
use std::fs::File;
use std::io::Write;
use std::io::{Read, Seek, SeekFrom};
use std::path::Path;
use std::path::PathBuf;

mod discord;
const MAXBUF: u64 = 8_380_416;

const HELP: &[&str] = &[
    concat!("+---- dimgup-cli ", env!("CARGO_PKG_VERSION"), " ---+"),
    "dimgup-cli config <webhook_url> - set webhook",
    "dimgup-cli <file> - upload file to discord",
    "dimgup-cli help - show this message",
];

macro_rules! loop_try {
    ($e:expr) => {
        match $e {
            Ok(v) => v,
            Err(_) => continue,
        }
    };
}

macro_rules! try_or_return {
    ($e:expr) => {
        match $e {
            Ok(v) => v,
            Err(_) => return,
        }
    };
}

fn get_config_file() -> PathBuf {
    let proj_dirs = ProjectDirs::from("xyz", "karx", "dimgup-cli").unwrap();
    let mut b = proj_dirs.config_dir().to_path_buf();
    b.push("webhook");
    b
}

fn get_webhook(path: &Path) -> anyhow::Result<String> {
    Ok(std::fs::read_to_string(path)?)
}

fn set_webhook(webhook: &str) -> anyhow::Result<()> {
    let proj_dirs = ProjectDirs::from("xyz", "karx", "dimgup-cli").unwrap();
    let mut b = proj_dirs.config_dir().to_path_buf();
    std::fs::create_dir_all(&b)?;
    b.push("webhook");
    let mut file = File::create(&b)?;
    file.write_all(webhook.as_bytes())?;
    Ok(())
}

fn get_file_info(path: &Path) -> anyhow::Result<(File, u64, String)> {
    let filename = path.file_name().unwrap().to_str().unwrap();
    let mut handle = File::open(path)?;
    let filesize = handle.seek(SeekFrom::End(0))?;
    handle.seek(SeekFrom::Start(0))?;
    if filesize > MAXBUF {
        bail!("File is too big.")
    } else {
        Ok((handle, filesize, filename.to_string()))
    }
}

fn do_upload(mut file: File, size: u64, name: &str, webhook: &str) -> anyhow::Result<String> {
    let mut buffer = vec![0u8; size as usize];
    file.read_exact(&mut *buffer)?;
    let progress_update = |bytes: f64| {
        let chars = ['\\', '|', '/', '-'];
        print!("{} \r", chars[((bytes * 100.0) as usize) % chars.len()]);
        try_or_return!(std::io::stdout().flush());
    };
    discord::upload(name, &buffer, webhook, progress_update)
}

fn main() -> anyhow::Result<()> {
    let filename = std::env::args()
        .nth(1)
        .ok_or_else(|| anyhow!("No filename or argument was provided"))?;
    if filename == "config" {
        let webhook = std::env::args()
            .nth(2)
            .ok_or_else(|| anyhow!("No webhook was provided"))?;
        set_webhook(&webhook)?;
        println!("Webhook set successfully");
        return Ok(());
    } else if filename == "help" {
        println!("{}", HELP.join("\n"));
        return Ok(());
    }

    let conf = get_config_file();
    let webhook = match get_webhook(&conf) {
        Ok(w) => w,
        Err(_) => bail!("Please set your webhook. See `dimgup-cli help` for more info."),
    };
    let filename = PathBuf::from(filename);

    let (handle, size, name) = get_file_info(&filename)?;

    let url = do_upload(handle, size, &name, &webhook)?;

    println!("Upload finished! URL: {}", url);
    Ok(())
}
